@@ -73,9 +73,12 @@ ulimit -c unlimited
 # Force TTY output.
 export SYD_FORCE_TTY=YesPlease
 
-# Timeout is 20 minutes per-test,
-# unless otherwise specified.
-SYD_TEST_TIMEOUT=${SYD_TEST_TIMEOUT:-20m}
+# Enable path hiding and ioctl sandboxing for wider coverage.
+export SYD_PALUDIS_LPATH=1
+export SYD_PALUDIS_IOCTL=1
+
+# Timeout is 45 minutes for all tests unless otherwise specified.
+SYD_TEST_TIMEOUT=${SYD_TEST_TIMEOUT:-45m}
 
 export SYD_LOG=${SYD_LOG:-info}
 SYD="${CARGO_BIN_EXE_syd:-syd}"
