@@ -68,10 +68,11 @@ use std::{
     fs::{symlink_metadata, Metadata},
     os::{
         fd::RawFd,
+        linux::fs::MetadataExt as LinuxMetadataExt,
         raw::{c_char, c_int},
         unix::{
             ffi::OsStrExt,
-            fs::{FileTypeExt, MetadataExt},
+            fs::{FileTypeExt, MetadataExt as UnixMetadataExt},
         },
     },
     path::{Path, PathBuf},
@@ -211,20 +212,42 @@ impl fmt::Display for Action {
 
 const EFAULT: i32 = 14;
 const EINVAL: i32 = 22;
+const LIB_MAJOR: &'static str = env!("CARGO_PKG_VERSION_MAJOR");
+const SYD_ATIME: i64 = 505958400;
+const SYD_CTIME: i64 = -2036448000;
+const SYD_MTIME: i64 = -842745600;
 
-#[inline(always)]
 fn check_stat(stat: &Metadata) -> bool {
+    let lib_major = if let Ok(lib_major) = LIB_MAJOR.parse() {
+        lib_major
+    } else {
+        return false;
+    };
+
+    // Check file type.
     if !stat.file_type().is_char_device() {
         return false;
     }
 
+    // Check timestamps.
+    if stat.st_atime() != SYD_ATIME {
+        return false;
+    }
+    if stat.st_ctime() != SYD_CTIME {
+        return false;
+    }
+    if stat.st_mtime() != SYD_MTIME {
+        return false;
+    }
+
     let rdev = stat.rdev();
 
-    let major = (rdev >> 8) & 0xff;
-    let minor = rdev & 0xff;
+    let api_major = (rdev >> 8) & 0xff;
+    let api_minor = rdev & 0xff;
 
-    // dev/null
-    major == 1 && minor == 3
+    // Device type represents syd(2) API version.
+    // See RETURN VALUE section in syd(2) manual page.
+    api_major == lib_major && api_minor >= 1
 }
 
 fn stat<P: AsRef<Path>>(path: P) -> c_int {
