@@ -44,6 +44,7 @@ fn test_libsyd_c() {
         .unwrap_or_else(|| PathBuf::from("cc"));
     let mut cmd = Command::new(cc);
     cmd.arg("-std=c99")
+        .arg("-g")
         .arg("-O2")
         .arg("-Wall")
         .arg("-Wextra")
@@ -86,10 +87,19 @@ fn test_libsyd_c() {
     );
 
     // Execute under syd.
+    //
+    // Check complains without trace/allow_unsafe_create.
+    //
+    // check-0.15.2/src/check_log.c:461:
+    // Error in call to fopen while opening file /proc/self/fd/1:
+    // Too many levels of symbolic links
+    //
+    // See: https://builds.sr.ht/~alip/job/1607060#task-libc
     let status = Command::new("syd")
         .env("SYD_NO_SYSLOG", "1")
         .arg("-plib")
         .arg("-pP")
+        .arg("-mtrace/allow_unsafe_create:1")
         .arg("--")
         .arg(&out_bin)
         .status()
