@@ -15,7 +15,7 @@ use std::{
     fmt::Write as FmtWrite,
     fs::{metadata, File, OpenOptions},
     hash::{Hash, Hasher},
-    io::{self, stdin, BufRead, BufReader, Read, Write as IoWrite},
+    io::{self, stderr, stdin, BufRead, BufReader, Read, Write as IoWrite},
     iter::FromIterator,
     net::{IpAddr, SocketAddrV4, SocketAddrV6},
     os::{
@@ -158,20 +158,20 @@ type PandoraSet<K> = std::collections::HashSet<K, ahash::RandomState>;
 // write! which returns Errno.
 macro_rules! w {
     ($out:expr) => {
-        write!($out).or(Err(Errno::EIO))
+        retry_on_intr(|| write!($out).or(Err(Errno::EIO)))
     };
     ($out:expr, $($arg:tt)*) => {
-        write!($out, $($arg)*).or(Err(Errno::EIO))
+        retry_on_intr(|| write!($out, $($arg)*).or(Err(Errno::EIO)))
     };
 }
 
 // writeln! which returns Errno.
 macro_rules! wln {
     ($out:expr) => {
-        writeln!($out).map_err(err2no)
+        retry_on_intr(|| writeln!($out).map_err(err2no))
     };
     ($out:expr, $($arg:tt)*) => {
-        writeln!($out, $($arg)*).map_err(err2no)
+        retry_on_intr(|| writeln!($out, $($arg)*).map_err(err2no))
     };
 }
 
@@ -684,20 +684,23 @@ impl AccessMap {
             Ok(json) => json,
             Err(err) => {
                 if env::var_os("PANDORA_DEBUG").is_some() {
-                    eprintln!(
+                    let _ = wln!(
+                        stderr(),
                         "{} {}",
                         style("pandora:").bold().magenta(),
                         style("skip invalid JSON!").bold().yellow()
                     );
-                    eprintln!(
+                    let _ = wln!(
+                        stderr(),
                         "\t{} {}",
                         style("LINE:").bold().cyan(),
                         style(line).bold().red()
                     );
-                    eprintln!(
+                    let _ = wln!(
+                        stderr(),
                         "\t{} {}",
                         style("ERROR:").bold().cyan(),
-                        style(err).bold().red()
+                        style(err.to_string()).bold().red()
                     );
                 }
                 return None;
@@ -789,22 +792,26 @@ impl AccessMap {
 
 fn command_profile(opts: ProfileOpts) -> Result<ExitCode, Errno> {
     if Path::new(&opts.output).exists() {
-        eprintln!(
+        wln!(
+            stderr(),
             "{} error creating output file: `{}' already exists!",
             style("pandora:").bold().magenta(),
-            style(opts.output).bold().yellow(),
-        );
+            style(opts.output.clone()).bold().yellow(),
+        )
+        .unwrap();
         return Ok(ExitCode::from(1));
     }
 
     let (fd_rd, fd_rw) = match pipe2(OFlag::O_CLOEXEC) {
         Ok((fd_rd, fd_rw)) => (fd_rd, fd_rw),
         Err(error) => {
-            eprintln!(
+            wln!(
+                stderr(),
                 "{} error creating pipe: {}!",
                 style("pandora:").bold().magenta(),
-                style(format!("{error}")).bold().red()
-            );
+                style(error.to_string()).bold().red()
+            )
+            .unwrap();
             return Ok(ExitCode::from(1));
         }
     };
@@ -817,7 +824,8 @@ fn command_profile(opts: ProfileOpts) -> Result<ExitCode, Errno> {
     syd.env("SYD_LOG_FD", &log_fd);
     if env::var_os("PANDORA_DEBUG").is_some() {
         let ino = fstatx(&fd_rw, STATX_INO).map(|stx| stx.stx_ino)?;
-        eprintln!(
+        let _ = wln!(
+            stderr(),
             "{} syd log fd set to pipe {} with inode {}.",
             style("pandora:").bold().magenta(),
             style(&log_fd).bold().green(),
@@ -854,7 +862,8 @@ fn command_profile(opts: ProfileOpts) -> Result<ExitCode, Errno> {
             .name("pandora_mon".to_string())
             .spawn(move || {
                 thread::sleep(cmd_timeout);
-                eprintln!(
+                let _ = wln!(
+                    stderr(),
                     "{} {}",
                     style("pandora:").bold().magenta(),
                     style("Timeout expired, terminating process...")
@@ -1425,7 +1434,8 @@ fn do_inspect(
         wln!(&mut output, "allow/{}+{}", caps.join(","), elem)?;
     }
 
-    eprintln!(
+    let _ = wln!(
+        stderr(),
         "{} generated {} rules.",
         style("pandora:").bold().magenta(),
         style(len.to_string()).bold().yellow(),
@@ -1605,7 +1615,8 @@ fn do_inspect(
         }
         wln!(&mut output)?;
 
-        eprintln!(
+        let _ = wln!(
+            stderr(),
             "{} calculated {} checksums for {} executables.",
             style("pandora:").bold().magenta(),
             style(hash_function.to_string()).bold().cyan(),
@@ -1613,18 +1624,21 @@ fn do_inspect(
         );
     }
 
-    eprintln!(
+    let _ = wln!(
+        stderr(),
         "{} profile generation completed! \\o/",
         style("pandora:").bold().magenta(),
     );
 
-    eprintln!(
+    let _ = wln!(
+        stderr(),
         "{} profile has been written to `{}'.",
         style("pandora:").bold().magenta(),
         style(output_path).bold().yellow(),
     );
 
-    eprintln!(
+    let _ = wln!(
+        stderr(),
         "{} To use it, do: {} -P \"{}\" -- command args...",
         style("pandora:").bold().magenta(),
         style("syd").bold().green(),
@@ -1707,11 +1721,13 @@ fn open_input(input: &str) -> Box<dyn BufRead> {
         "dmesg" | "syslog" => Box::new(BufReader::new(match Syslog::open() {
             Ok(syslog) => syslog,
             Err(err) => {
-                eprintln!(
+                wln!(
+                    stderr(),
                     "{} failed to access syslog: {}!",
                     style("pandora:").bold().magenta(),
-                    style(format!("{err}")).bold().red(),
-                );
+                    style(err.to_string()).bold().red(),
+                )
+                .unwrap();
                 exit(1);
             }
         })),
@@ -1720,12 +1736,14 @@ fn open_input(input: &str) -> Box<dyn BufRead> {
             match OpenOptions::new().read(true).open(path) {
                 Ok(file) => file,
                 Err(err) => {
-                    eprintln!(
+                    wln!(
+                        stderr(),
                         "{} failed to open file {}: {}!",
                         style("pandora:").bold().magenta(),
                         style(path).bold().yellow(),
-                        style(err).bold().red(),
-                    );
+                        style(err.to_string()).bold().red(),
+                    )
+                    .unwrap();
                     exit(1);
                 }
             },
@@ -1742,12 +1760,14 @@ fn open_output(path_or_stdout: &str) -> Box<dyn std::io::Write> {
             match OpenOptions::new().write(true).create_new(true).open(path) {
                 Ok(file) => file,
                 Err(err) => {
-                    eprintln!(
+                    wln!(
+                        stderr(),
                         "{} failed to open file {}: {}!",
                         style("pandora:").bold().magenta(),
                         style(path).bold().cyan(),
-                        style(err).bold().red(),
-                    );
+                        style(err.to_string()).bold().red(),
+                    )
+                    .unwrap();
                     exit(1);
                 }
             },
@@ -2002,7 +2022,8 @@ fn cap2prio(cap: &String) -> usize {
 #[expect(clippy::disallowed_methods)]
 fn print_help_main() {
     let nproc = env::var("PANDORA_NPROC").unwrap();
-    eprint!(
+    w!(
+        stderr(),
         r#"{PKG_NAME} {PKG_VERSION}
 {PKG_DESCRIPTION}
 Copyright (c) 2023, 2024, 2025 {PKG_AUTHORS}
@@ -2038,13 +2059,14 @@ Attaching poems encourages consideration tremendously.
 Homepage: https://sydbox.exherbo.org
 Repository: https://gitlab.exherbo.org/sydbox/
 "#,
-    );
+    ).unwrap();
 }
 
 #[expect(clippy::disallowed_methods)]
 fn print_help_profile() {
     let nproc = env::var("PANDORA_NPROC").unwrap();
-    eprint!(
+    w!(
+        stderr(),
         r#"{PKG_NAME} {PKG_VERSION}
 Profile subcommand
 
@@ -2062,13 +2084,14 @@ Options:
                sha3-512 (default), sha3-384, sha3-256, sha1, md5, crc64, crc32
   -T <threads> Number of concurrency threads used for parallel hashing [default: {nproc}, env:PANDORA_NPROC]
 "#,
-    );
+    ).unwrap();
 }
 
 #[expect(clippy::disallowed_methods)]
 fn print_help_inspect() {
     let nproc = env::var("PANDORA_NPROC").unwrap();
-    eprint!(
+    w!(
+        stderr(),
         r#"{PKG_NAME} {PKG_VERSION}
 Inspect subcommand
 
@@ -2084,11 +2107,11 @@ Options:
                sha3-512 (default), sha3-384, sha3-256, sha1, md5, crc64, crc32
   -T <threads> Number of concurrency threads used for parallel hashing [default: {nproc}, env:PANDORA_NPROC]
 "#,
-    );
+    ).unwrap();
 }
 
 fn print_version() {
-    eprintln!("{PKG_NAME}-{PKG_VERSION}");
+    wln!(stderr(), "{PKG_NAME}-{PKG_VERSION}").unwrap();
 }
 
 // Returns the longest prefix of `s` shared by ≥2 keys in `set`, else `s`.
@@ -2175,7 +2198,7 @@ fn parse_utf8_str(bytes: &[u8]) -> Result<String, Errno> {
     match std::str::from_utf8(bytes) {
         Ok(s) => Ok(s.to_owned()),
         Err(_) => {
-            eprintln!("ERROR: invalid UTF-8 data");
+            wln!(stderr(), "ERROR: invalid UTF-8 data")?;
             Err(Errno::EINVAL)
         }
     }
@@ -2343,3 +2366,20 @@ fn fstatx<Fd: AsFd>(fd: Fd, mask: libc::c_uint) -> Result<FileStatx, Errno> {
 
 // Want/got stx_ino.
 const STATX_INO: libc::c_uint = 0x00000100;
+
+// Retries a closure on `EAGAIN` and `EINTR` errors.
+//
+// This function will call the provided closure, and if the closure
+// returns `EAGAIN` or `EINTR` error, it will retry the operation until it
+// succeeds or fails with a different error.
+fn retry_on_intr<F, T>(mut f: F) -> Result<T, Errno>
+where
+    F: FnMut() -> Result<T, Errno>,
+{
+    loop {
+        match f() {
+            Err(Errno::EAGAIN | Errno::EINTR) => continue,
+            result => return result,
+        }
+    }
+}
