@@ -630,6 +630,24 @@ pub static API_SPEC: LazyLock<Api> = LazyLock::new(|| Api {
             op: Some(Operator::Set),
             ..Default::default()
         },
+        Method {
+            name: "force_no_symlinks".into(),
+            desc: "Force RESOLVE_NO_SYMLINKS at open(2) boundary".into(),
+            path: Some("trace/force_no_symlinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
+        Method {
+            name: "force_no_magiclinks".into(),
+            desc: "Force RESOLVE_NO_MAGICLINKS at open(2) boundary".into(),
+            path: Some("trace/force_no_magiclinks".into()),
+            argv: Some(vec![Arg::Boolean]),
+            argc: Some(vec![1]),
+            op: Some(Operator::Set),
+            ..Default::default()
+        },
         Method {
             name: "force_no_xdev".into(),
             desc: "Force RESOLVE_NO_XDEV at open(2) boundary".into(),
@@ -913,7 +931,6 @@ impl Serialize for Api {
 }
 
 // A single `/dev/syd` command binding.
-#[allow(clippy::type_complexity)]
 #[derive(Debug, Default)]
 struct Method {
     // Method name (e.g. `enable_stat`).
