@@ -35,7 +35,7 @@ use std::{
 use memchr::arch::all::{is_equal, is_prefix};
 use nix::errno::Errno;
 
-use crate::{fs::safe_open_file, XPath};
+use crate::{lookup::safe_open_file, XPath};
 
 /// ELF magic number used to identify ELF files.
 const ELF_MAGIC: &[u8] = b"\x7FELF";
