@@ -119,7 +119,7 @@ pub(crate) fn sys_fanotify_mark(request: UNotifyEventRequest) -> ScmpNotifResp {
             // SAFETY: We open a FD to the path and then use the
             // proc path $PROC_FILE/thread-self/fd/$fd in address'
             // path argument to avoid symlink TOCTOU.
-            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
+            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd())?;
 
             // Validate the FANotify FD.
             let notify_fd = RawFd::try_from(req.data.args[0]).or(Err(Errno::EBADF))?;
