@@ -10,10 +10,10 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
-    fs::FsFlags,
-    ioctl::{ioctl_names_get, Ioctl},
+    ioctl::Ioctl,
     kernel::sandbox_path,
     log_enabled,
+    lookup::FsFlags,
     req::{SysArg, UNotifyEventRequest},
     sandbox::{Action, Capability},
     syslog::LogLevel,
@@ -34,7 +34,7 @@ pub(crate) fn sys_ioctl(request: UNotifyEventRequest) -> ScmpNotifResp {
             fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
             ..Default::default()
         };
-        let (path, _, _) = request.read_path(&sandbox, arg, false)?;
+        let (path, _, _) = request.read_path(&sandbox, arg)?;
 
         // Check if the ioctl(2) request is allowlisted or denylisted.
         let arg = req.data.args[1];
@@ -82,14 +82,13 @@ pub(crate) fn sys_ioctl(request: UNotifyEventRequest) -> ScmpNotifResp {
             // _ioctl_(2) request is denylisted.
             let cap = Capability::CAP_IOCTL;
             let action = sandbox.default_action(cap);
-            let verbose = sandbox.verbose;
-            drop(sandbox); // release the read-lock.
 
             let filter = action == Action::Filter;
             if !filter && action >= Action::Warn && log_enabled!(LogLevel::Warn) {
-                let ctl = ioctl_names_get(arg as Ioctl, req.data.arch);
+                let log_scmp = sandbox.log_scmp();
+                let ctl = sandbox.get_ioctl_names(arg as Ioctl, req.data.arch);
                 let grp = cap.to_string().to_ascii_lowercase();
-                if verbose {
+                if log_scmp {
                     warn!("ctx": "access", "cap": cap, "act": action,
                         "sys": "ioctl", "ctl": ctl,
                         "tip": format!("configure `{grp}/allow+{arg:#x}'"),
@@ -101,6 +100,7 @@ pub(crate) fn sys_ioctl(request: UNotifyEventRequest) -> ScmpNotifResp {
                         "pid": request.scmpreq.pid);
                 }
             }
+            drop(sandbox); // release the read-lock.
 
             return match action {
                 Action::Allow | Action::Warn => {
