@@ -132,7 +132,7 @@ fn syscall_mem_handler(
 
     // Get mem & vm max.
     let sandbox = request.get_sandbox();
-    let verbose = sandbox.verbose;
+    let log_scmp = sandbox.log_scmp();
     let caps = sandbox.getcaps(caps);
     let exec = caps.contains(Capability::CAP_EXEC);
     let force = caps.contains(Capability::CAP_FORCE);
@@ -248,7 +248,7 @@ fn syscall_mem_handler(
             let (action, msg) = sandbox.check_tpe(file, path.abs());
             if !matches!(action, Action::Allow | Action::Filter) {
                 let msg = msg.as_deref().unwrap_or("?");
-                if verbose {
+                if log_scmp {
                     error!("ctx": "trusted_path_execution",
                         "msg": format!("library load from untrusted path blocked: {msg}"),
                         "sys": &name, "path": &path,
@@ -306,7 +306,7 @@ fn syscall_mem_handler(
                     let exe = ExecutableFile::parse(&mut *file, true).or(Err(Errno::EACCES))?;
                     if matches!(exe, ExecutableFile::Elf { xs: true, .. }) {
                         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                            if verbose {
+                            if log_scmp {
                                 error!("ctx": "check_lib",
                                     "msg": "library load with executable stack blocked",
                                     "sys": &name, "path": path.abs(),
@@ -344,7 +344,7 @@ fn syscall_mem_handler(
                 let deny = match result {
                     Ok(action) => {
                         if !matches!(action, Action::Allow | Action::Filter) {
-                            if verbose {
+                            if log_scmp {
                                 warn!("ctx": "verify_lib", "act": action,
                                     "sys": &name, "path": path.abs(),
                                     "tip": format!("configure `force+{}:<checksum>'", path.abs()),
@@ -369,7 +369,7 @@ fn syscall_mem_handler(
                         }
                     }
                     Err(IntegrityError::Sys(errno)) => {
-                        if verbose {
+                        if log_scmp {
                             error!("ctx": "verify_lib",
                                 "msg": format!("system error during library checksum calculation: {errno}"),
                                 "sys": &name, "path": path.abs(),
@@ -390,7 +390,7 @@ fn syscall_mem_handler(
                         found,
                     }) => {
                         if action != Action::Filter {
-                            if sandbox.verbose {
+                            if log_scmp {
                                 error!("ctx": "verify_lib", "act": action,
                                     "msg": format!("library checksum mismatch: {found} is not {expected}"),
                                     "sys": &name, "path": path.abs(),
@@ -447,7 +447,7 @@ fn syscall_mem_handler(
         };
         if mem_vm_cur.saturating_add(size) >= mem_vm_max {
             if mem_act != Action::Filter {
-                if verbose {
+                if log_scmp {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
                         "sys": &name, "mem_vm_max": mem_vm_max, "mem_vm_cur": mem_vm_cur,
                         "mem_size": size, "tip": "increase `mem/vm_max'",
@@ -482,7 +482,7 @@ fn syscall_mem_handler(
         };
         if mem_cur.saturating_add(size) >= mem_max {
             if mem_act != Action::Filter {
-                if verbose {
+                if log_scmp {
                     warn!("ctx": "access", "cap": Capability::CAP_MEM, "act": mem_act,
                         "sys": &name, "mem_max": mem_max, "mem_cur": mem_cur,
                         "mem_size": size, "tip": "increase `mem/max'",
