@@ -55,14 +55,15 @@ pub(crate) fn sys_memfd_create(request: UNotifyEventRequest) -> ScmpNotifResp {
             flags.insert(MFdFlags::MFD_NOEXEC_SEAL);
         }
 
-        let mut buf = Vec::new();
-        buf.try_reserve(MFD_NAME_MAX_LEN).or(Err(Errno::ENOMEM))?;
-        buf.resize(MFD_NAME_MAX_LEN, 0);
-        let mut n = request.read_mem(&mut buf, addr)?;
-        if let Some(null) = memchr(0, &buf[..n]) {
-            n = n.min(null.checked_add(1).ok_or(Errno::EOVERFLOW)?);
-        }
-        buf.truncate(n);
+        let mut buf = request.read_vec(addr, MFD_NAME_MAX_LEN)?;
+
+        // If buffer has no null byte,
+        // return EINVAL as the path is too long for us to handle.
+        let null = memchr(0, &buf)
+            .ok_or(Errno::EINVAL)?
+            .checked_add(1)
+            .ok_or(Errno::EINVAL)?;
+        buf.truncate(null);
         buf.shrink_to_fit();
 
         // SAFETY:
@@ -73,9 +74,9 @@ pub(crate) fn sys_memfd_create(request: UNotifyEventRequest) -> ScmpNotifResp {
             buf[0] = b'S';
         }
 
-        // If buffer has no null byte,
-        // return EINVAL as the path is too long for us to handle.
-        let name = CString::from_vec_with_nul(buf).or(Err(Errno::EINVAL))?;
+        // SAFETY:
+        // Buffer is guaranteed to have null-byte as its last element.
+        let name = unsafe { CString::from_vec_with_nul_unchecked(buf) };
 
         // Determine sandbox capabilities.
         let mut caps = Capability::CAP_CREATE;
