@@ -135,10 +135,7 @@ pub(crate) mod xattr;
 /// ptrace(2) hooks
 pub(crate) mod ptrace;
 
-use std::borrow::Cow;
-
 use libseccomp::ScmpNotifResp;
-use memchr::memmem;
 use nix::{
     errno::Errno,
     fcntl::{AtFlags, OFlag},
@@ -151,13 +148,15 @@ use nix::{
 
 use crate::{
     fs::{fd_status_flags, to_valid_fd},
-    ioctl::{ioctl_names_get, Ioctl},
+    ioctl::Ioctl,
     log::log_is_main,
+    log_enabled,
     lookup::{CanonicalPath, FileInfo, FileType},
     notice,
-    path::{XPath, XPathBuf},
+    path::XPath,
     req::{PathArgs, SysArg, UNotifyEventRequest},
     sandbox::{Action, Capability, Sandbox, SandboxGuard},
+    syslog::LogLevel,
     warn,
 };
 
@@ -191,21 +190,6 @@ pub(crate) fn sandbox_path(
         };
     }
 
-    // Convert /proc/${pid} to /proc/self as necessary.
-    let path = if let Some(p) = path.split_prefix(b"/proc") {
-        let mut buf = itoa::Buffer::new();
-        let pid = buf.format(pid.as_raw());
-        if let Some(p) = p.split_prefix(pid.as_bytes()) {
-            let mut pdir = XPathBuf::from("/proc/self");
-            pdir.push(p.as_bytes());
-            Cow::Owned(pdir)
-        } else {
-            Cow::Borrowed(path)
-        }
-    } else {
-        Cow::Borrowed(path)
-    };
-
     let mut action = Action::Allow;
     let mut filter = false;
     let check_caps = caps & Capability::CAP_PATH;
@@ -217,7 +201,7 @@ pub(crate) fn sandbox_path(
 
     // Sandboxing.
     for cap in check_caps {
-        let (new_action, new_filter) = sandbox.check_path(cap, &path);
+        let (new_action, new_filter) = sandbox.check_path(cap, path);
 
         if new_action >= action {
             action = new_action;
@@ -229,7 +213,7 @@ pub(crate) fn sandbox_path(
     // for sandboxing types other than Stat.
     let check_hidden = stat && hide && (caps.is_empty() || action.is_denying());
     if check_hidden || caps.contains(Capability::CAP_STAT) {
-        let (new_action, new_filter) = sandbox.check_path(Capability::CAP_STAT, &path);
+        let (new_action, new_filter) = sandbox.check_path(Capability::CAP_STAT, path);
 
         if !check_hidden {
             deny_errno = Errno::ENOENT;
@@ -250,7 +234,7 @@ pub(crate) fn sandbox_path(
         }
     }
 
-    if !filter && action >= Action::Warn {
+    if !filter && action >= Action::Warn && log_enabled!(LogLevel::Warn) {
         // Log warn for normal cases.
         // Log info for path hiding/walking unless explicitly specified to warn.
         let is_warn = match caps {
@@ -271,12 +255,12 @@ pub(crate) fn sandbox_path(
 
             // Enrich log data with ioctl(2) request names.
             let ctl = if caps.contains(Capability::CAP_IOCTL) {
-                ioctl_names_get(args[1] as Ioctl, arch)
+                sandbox.get_ioctl_names(args[1] as Ioctl, arch)
             } else {
                 None
             };
 
-            if sandbox.verbose {
+            if sandbox.log_scmp() {
                 if is_warn {
                     warn!("ctx": "access", "cap": caps, "act": action,
                         "sys": syscall_name, "ctl": ctl,
@@ -330,7 +314,7 @@ pub(crate) fn sandbox_path(
 
     match action {
         Action::Allow | Action::Warn => {
-            if caps.can_write() && sandbox.is_append(&path) {
+            if caps.can_write() && sandbox.is_append(path) {
                 // SAFETY: Protect append-only paths against writes.
                 // We use ECANCELED which will result in a no-op.
                 Err(Errno::ECANCELED)
@@ -414,17 +398,14 @@ where
         let crypt = sandbox.enabled(Capability::CAP_CRYPT);
         let hide = !is_fd && sandbox.enabled(Capability::CAP_STAT);
 
-        // EXCEPTION: We do want to return success to _access_(2) calls
-        // to magic paths in case the sandbox lock allows it.
-        let mut magic = !is_fd && memmem::find(syscall_name.as_bytes(), b"access").is_some();
-
+        let mut magic = false;
         let mut paths: [Option<CanonicalPath>; 2] = [None, None];
         for (idx, arg) in path_argv.iter().enumerate() {
             // Handle system calls that take a FD only,
             // such as fchmod, fchown, falllocate, ftruncate,
             // fgetxattr, fsetxattr safely and efficiently.
             if arg.path.is_some() {
-                let (path, is_magic, _) = request.read_path(&sandbox, *arg, magic)?;
+                let (path, is_magic, _) = request.read_path(&sandbox, *arg)?;
                 magic = is_magic;
 
                 if sandbox.is_chroot() {
