@@ -27,7 +27,7 @@ use zeroize::Zeroizing;
 use crate::{
     compat::{
         cmsg_len_32, cmsg_space_32, cmsghdr, cmsghdr32, iovec32, mmsghdr, mmsghdr32, mmsghdr_union,
-        msghdr, msghdr32, MsgFlags,
+        msghdr, msghdr32, MsgFlags, UIO_MAXIOV, UIO_MAXLEN,
     },
     config::MMAP_MIN_ADDR,
     confine::scmp_arch_bits,
@@ -66,18 +66,15 @@ pub(crate) fn handle_recvmsg(
     } else {
         size_of::<msghdr>()
     };
-    let mut hdr_buf = zeroize::Zeroizing::new(Vec::new());
-    hdr_buf.try_reserve(hdr_sz).or(Err(Errno::ENOMEM))?;
-    hdr_buf.resize(hdr_sz, 0);
-    request.read_mem(&mut hdr_buf, args[1])?;
 
+    let hdr = request.read_vec_zeroed(args[1], hdr_sz)?;
     let mut hdr: msghdr = if is32 {
         // SAFETY: Unaligned read of POD; size validated above.
-        let m32: msghdr32 = unsafe { std::ptr::read_unaligned(hdr_buf.as_ptr() as *const _) };
+        let m32: msghdr32 = unsafe { std::ptr::read_unaligned(hdr.as_ptr() as *const msghdr32) };
         msghdr::from(m32)
     } else {
         // SAFETY: Unaligned read of POD; size validated above.
-        unsafe { std::ptr::read_unaligned(hdr_buf.as_ptr() as *const _) }
+        unsafe { std::ptr::read_unaligned(hdr.as_ptr() as *const msghdr) }
     };
 
     // Mirror sandbox process iovecs to local, bounded buffers.
@@ -412,7 +409,7 @@ fn handle_recvmmsg_internal(
     //
     // 1. Validate message count.
     let msg_count = usize::try_from(args[2]).or(Err(Errno::EINVAL))?;
-    if msg_count > 1_000_000 {
+    if msg_count > UIO_MAXLEN {
         // Invalid message count: Too large.
         return Err(Errno::EMSGSIZE);
     }
@@ -424,12 +421,8 @@ fn handle_recvmmsg_internal(
     } else {
         size_of::<mmsghdr>()
     };
-
-    let mut hdr_buf = Zeroizing::new(Vec::new());
     let total_sz = hdr_sz.checked_mul(msg_count).ok_or(Errno::EOVERFLOW)?;
-    hdr_buf.try_reserve(total_sz).or(Err(Errno::ENOMEM))?;
-    hdr_buf.resize(total_sz, 0);
-    request.read_mem(&mut hdr_buf, msgs_offset)?;
+    let hdr = request.read_vec_zeroed(msgs_offset, total_sz)?;
 
     // 3. Convert to native format to pass to recvmmsg(2).
     let mut msgs = Vec::new();
@@ -456,7 +449,7 @@ fn handle_recvmmsg_internal(
         .try_reserve(msg_count)
         .or(Err(Errno::ENOMEM))?;
 
-    for chunk in hdr_buf.chunks(hdr_sz) {
+    for chunk in hdr.chunks(hdr_sz) {
         let mut hdr: libc::mmsghdr = if is32 {
             // SAFETY: We know that hdr_buf was allocated with enough space to hold mmsghdr32.
             let m32: mmsghdr32 = unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) };
@@ -807,7 +800,7 @@ fn process_mmsghdr_iov(
         // Invalid iovec buffer.
         return Err(Errno::EFAULT);
     }
-    if !msg_iov.is_null() && iov_len > 1024 {
+    if !msg_iov.is_null() && iov_len > UIO_MAXIOV {
         // Invalid iovec count: Too large or negative.
         return Err(Errno::EMSGSIZE);
     }
@@ -815,10 +808,7 @@ fn process_mmsghdr_iov(
     // Handle msg_iov.
     if !msg_iov.is_null() && iov_len > 0 {
         let iov_siz = iov_len.checked_mul(vec_siz).ok_or(Errno::EINVAL)?;
-        let mut iov_raw = Vec::new();
-        iov_raw.try_reserve(iov_siz).or(Err(Errno::ENOMEM))?;
-        iov_raw.resize(iov_siz, 0);
-        request.read_mem(&mut iov_raw, msg_iov as u64)?;
+        let iov_raw = request.read_vec(msg_iov as u64, iov_siz)?;
 
         #[expect(clippy::type_complexity)]
         let mut bufs: Vec<(Zeroizing<Vec<u8>>, u64)> = Vec::new();
@@ -846,7 +836,7 @@ fn process_mmsghdr_iov(
 
             if !iov.iov_base.is_null() && iov.iov_len > 0 {
                 // SAFETY: Cap untrusted `iov_len`.
-                let len = iov.iov_len.min(1_000_000);
+                let len = iov.iov_len.min(UIO_MAXLEN);
                 let mut buf = Vec::new();
                 buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
                 buf.resize(len, 0);
@@ -942,8 +932,8 @@ fn process_mmsghdr_ctl(
     #[expect(clippy::useless_conversion)]
     if !hdr.msg_hdr.msg_control.is_null() && hdr.msg_hdr.msg_controllen > 0 {
         let mut ctl = Vec::new();
-        // SAFETY: Cap length at 1 mio.
-        let mut len = hdr.msg_hdr.msg_controllen.min(1_000_000) as usize;
+        // SAFETY: Cap length at UIO_MAXLEN.
+        let mut len = (hdr.msg_hdr.msg_controllen as usize).min(UIO_MAXLEN);
         if is32 {
             // SAFETY: CMSG_SPACE is always safe.
             len = unsafe { libc::CMSG_SPACE(len as u32) } as usize;
@@ -997,7 +987,7 @@ fn process_msghdr_iov(
         // Invalid iovec buffer.
         return Err(Errno::EFAULT);
     }
-    if !msg_iov.is_null() && iov_len > 1024 {
+    if !msg_iov.is_null() && iov_len > UIO_MAXIOV {
         // Invalid iovec count: Too large or negative.
         return Err(Errno::EMSGSIZE);
     }
@@ -1005,10 +995,7 @@ fn process_msghdr_iov(
     // Handle msg_iov.
     if !msg_iov.is_null() && iov_len > 0 {
         let iov_siz = iov_len.checked_mul(vec_siz).ok_or(Errno::EINVAL)?;
-        let mut iov_raw = Zeroizing::new(Vec::new());
-        iov_raw.try_reserve(iov_siz).or(Err(Errno::ENOMEM))?;
-        iov_raw.resize(iov_siz, 0);
-        request.read_mem(&mut iov_raw, msg_iov as u64)?;
+        let iov_raw = request.read_vec_zeroed(msg_iov as u64, iov_siz)?;
 
         msg_bufs.try_reserve(iov_len).or(Err(Errno::ENOMEM))?;
         msg_iovs.try_reserve(iov_len).or(Err(Errno::ENOMEM))?;
@@ -1033,7 +1020,7 @@ fn process_msghdr_iov(
 
             if !iov.iov_base.is_null() && iov.iov_len > 0 {
                 // SAFETY: Cap untrusted `iov_len`.
-                let len = iov.iov_len.min(1_000_000);
+                let len = iov.iov_len.min(UIO_MAXLEN);
                 let mut buf = Vec::new();
                 buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
                 buf.resize(len, 0);
@@ -1109,8 +1096,8 @@ fn process_msghdr_ctl(
     }
     if !hdr.msg_control.is_null() && hdr.msg_controllen > 0 {
         let mut ctl = Vec::new();
-        // SAFETY: Cap length at 1 mio.
-        let mut len = hdr.msg_controllen.min(1_000_000);
+        // SAFETY: Cap length at UIO_MAXLEN.
+        let mut len = hdr.msg_controllen.min(UIO_MAXLEN);
         #[expect(clippy::cast_possible_truncation)]
         if is32 {
             // SAFETY: CMSG_SPACE is always safe.
