@@ -31,8 +31,9 @@ use crate::{
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err::err2no,
     error,
-    fs::{safe_open, AT_BADFD},
+    fs::AT_BADFD,
     log_enabled,
+    lookup::safe_open,
     proc::{proc_executables, proc_set_at_secure, SydExecMap},
     retry::retry_on_eintr,
     sandbox::{Action, Capability, IntegrityError, Sandbox, SandboxGuard},
