@@ -92,7 +92,7 @@ pub(crate) fn sysenter_exec(
     let process = RemoteProcess::new(pid);
 
     // SAFETY: This is a ptrace hook, the PID cannot be validated.
-    let (mut path, _, _, empty_path) = process.read_path(sandbox, data.args, arg, false, None)?;
+    let (mut path, _, _, empty_path) = process.read_path(sandbox, data.args, arg, None)?;
 
     // Call sandbox access checker if Exec sandboxing is on.
     let caps = sandbox.getcaps(Capability::CAP_EXEC | Capability::CAP_TPE);
