@@ -9,9 +9,8 @@
 use libseccomp::ScmpNotifResp;
 use nix::{errno::Errno, sys::signal::SaFlags};
 
-use crate::{config::MMAP_MIN_ADDR, hook::UNotifyEventRequest, proc::proc_tgid};
+use crate::{config::MMAP_MIN_ADDR, proc::proc_tgid, req::UNotifyEventRequest};
 
-#[allow(clippy::cognitive_complexity)]
 pub(crate) fn sys_sigaction(request: UNotifyEventRequest) -> ScmpNotifResp {
     // Check if the handler is a restarting one.
     // This allows us to selectively unblock system calls.
@@ -51,7 +50,12 @@ pub(crate) fn sys_sigaction(request: UNotifyEventRequest) -> ScmpNotifResp {
     };
 
     let _is_restart = if sa_flags.contains(SaFlags::SA_RESTART) {
-        request.cache.add_sig_restart(tgid, sig_num);
+        if let Err(errno) = request.cache.add_sig_restart(tgid, sig_num) {
+            // This may only fail under memory-pressure.
+            // Better to be on the safe side and deny the syscall.
+            // TODO: Log an alert here.
+            return request.fail_syscall(errno);
+        }
         true
     } else {
         request.cache.del_sig_restart(tgid, sig_num);
