@@ -9,15 +9,19 @@
 use std::{
     fs::File,
     io::BufReader,
-    os::fd::{AsFd, AsRawFd},
+    os::{fd::AsFd, unix::ffi::OsStrExt},
 };
 
+use data_encoding::HEXLOWER;
 use libseccomp::ScmpNotifResp;
 use nix::{errno::Errno, fcntl::AtFlags, NixPath};
 
 use crate::{
-    compat::{fstatat64, statx, STATX_BASIC_STATS, STATX_MODE, STATX_TYPE},
-    config::{MAGIC_PREFIX, MMAP_MIN_ADDR},
+    compat::{
+        fstatat64, statx, FileStat64, FileStatx, FileStatxTimestamp, STATX_BASIC_STATS, STATX_MODE,
+        STATX_TYPE,
+    },
+    config::{API_VERSION, ENV_ID, MAGIC_PREFIX, MMAP_MIN_ADDR},
     confine::{scmp_arch_bits, EOWNERDEAD},
     fs::parse_fd,
     hash::SydHashSet,
@@ -25,7 +29,7 @@ use crate::{
     lookup::{CanonicalPath, FileInfo, FileType, FsFlags},
     path::XPath,
     req::{SysArg, SysFlags, UNotifyEventRequest},
-    sandbox::Capability,
+    sandbox::{Capability, Flags},
 };
 
 pub(crate) fn sys_stat(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -40,6 +44,7 @@ pub(crate) fn sys_stat(request: UNotifyEventRequest) -> ScmpNotifResp {
 
     let arg = SysArg {
         path: Some(0),
+        flags: SysFlags::CHECK_MAGIC,
         fsflags: FsFlags::MUST_PATH,
         ..Default::default()
     };
@@ -57,6 +62,7 @@ pub(crate) fn sys_stat64(request: UNotifyEventRequest) -> ScmpNotifResp {
 
     let arg = SysArg {
         path: Some(0),
+        flags: SysFlags::CHECK_MAGIC,
         fsflags: FsFlags::MUST_PATH,
         ..Default::default()
     };
@@ -110,6 +116,7 @@ pub(crate) fn sys_lstat(request: UNotifyEventRequest) -> ScmpNotifResp {
 
     let arg = SysArg {
         path: Some(0),
+        flags: SysFlags::CHECK_MAGIC,
         fsflags: FsFlags::MUST_PATH | FsFlags::NO_FOLLOW_LAST,
         ..Default::default()
     };
@@ -127,6 +134,7 @@ pub(crate) fn sys_lstat64(request: UNotifyEventRequest) -> ScmpNotifResp {
 
     let arg = SysArg {
         path: Some(0),
+        flags: SysFlags::CHECK_MAGIC,
         fsflags: FsFlags::MUST_PATH | FsFlags::NO_FOLLOW_LAST,
         ..Default::default()
     };
@@ -162,6 +170,8 @@ pub(crate) fn sys_statx(request: UNotifyEventRequest) -> ScmpNotifResp {
     let mut fsflags = FsFlags::MUST_PATH;
     if atflags.contains(AtFlags::AT_EMPTY_PATH) {
         flags |= SysFlags::EMPTY_PATH;
+    } else {
+        flags |= SysFlags::CHECK_MAGIC;
     }
     if atflags.contains(AtFlags::AT_SYMLINK_NOFOLLOW) {
         fsflags |= FsFlags::NO_FOLLOW_LAST;
@@ -199,6 +209,8 @@ pub(crate) fn sys_newfstatat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let mut fsflags = FsFlags::MUST_PATH;
     if atflags.contains(AtFlags::AT_EMPTY_PATH) {
         flags |= SysFlags::EMPTY_PATH;
+    } else {
+        flags |= SysFlags::CHECK_MAGIC;
     }
     if atflags.contains(AtFlags::AT_SYMLINK_NOFOLLOW) {
         fsflags |= FsFlags::NO_FOLLOW_LAST;
@@ -224,23 +236,25 @@ fn syscall_stat_handler(
 ) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
-        let is_fd = arg.path.is_none();
         let sandbox = request.get_sandbox();
 
         let has_crypt = sandbox.enabled(Capability::CAP_CRYPT);
         let has_stat = sandbox.enabled(Capability::CAP_STAT);
 
+        // Read the remote path and check for magic path as necessary.
+        let (mut path, magic, empty_path) = request.read_path(&sandbox, arg)?;
+        let is_fd = empty_path || arg.path.is_none();
+
         // Check for chroot.
         if sandbox.is_chroot() {
             return Err(if is_fd { Errno::EACCES } else { Errno::ENOENT });
         }
 
-        // Read the remote path and check for magic path.
-        let (mut path, magic, _) = request.read_path(&sandbox, arg, true)?;
-
         let restrict_stat_bdev = !sandbox.flags.allow_unsafe_stat_bdev();
         let restrict_stat_cdev = !sandbox.flags.allow_unsafe_stat_cdev();
         let mut ghost = false;
+        let caps = *sandbox.state;
+        let sflags = *sandbox.flags;
         if magic {
             if sandbox.locked_for(req.pid()) {
                 // Sandbox is locked, access denied.
@@ -410,7 +424,7 @@ fn syscall_stat_handler(
             }
 
             // Note, unlike statfs, stat does not EINTR.
-            let mut statx = statx(fd, "", flags, mask)?;
+            let mut statx = statx(fd, c"", flags, mask)?;
 
             // SAFETY: Check if the file is a sidechannel device and
             // update its access and modification times to match the
@@ -439,6 +453,11 @@ fn syscall_stat_handler(
                 }
             }
 
+            // If magic path, mask values for easy id.
+            if magic {
+                magic_statx(&mut statx, caps, sflags);
+            }
+
             // SAFETY: The following block creates an immutable byte
             // slice representing the memory of `statx`. We ensure that
             // the slice covers the entire memory of `statx` using
@@ -463,7 +482,7 @@ fn syscall_stat_handler(
 
             // SAFETY: In libc we trust.
             // Note, unlike statfs, stat does not EINTR.
-            let mut stat = fstatat64(Some(fd.as_raw_fd()), "", flags)?;
+            let mut stat = fstatat64(fd, c"", flags)?;
 
             // SAFETY: Check if the file is a sidechannel device and
             // update its access and modification times to match the
@@ -481,6 +500,11 @@ fn syscall_stat_handler(
                 }
             }
 
+            // If magic path, mask values for easy id.
+            if magic {
+                magic_stat(&mut stat, caps, sflags);
+            }
+
             let addr = req.data.args[arg_stat];
             if addr != 0 {
                 if is32 {
@@ -536,3 +560,105 @@ fn syscall_stat_handler(
         Ok(request.return_syscall(0))
     })
 }
+
+fn magic_stat(stat: &mut FileStat64, caps: Capability, flags: Flags) {
+    stat.st_ino = magic_ino();
+    stat.st_nlink = caps.nlink().into();
+    stat.st_mode = magic_mode(caps, flags).into();
+    stat.st_rdev = API_VERSION.dev();
+    stat.st_atime = 505958400; // must match libsyd!
+    stat.st_ctime = -2036448000; // ditto!
+    stat.st_mtime = -842745600; // ditto!
+}
+
+fn magic_statx(statx: &mut FileStatx, caps: Capability, flags: Flags) {
+    statx.stx_ino = magic_ino();
+    statx.stx_nlink = caps.nlink();
+    statx.stx_mode = magic_mode(caps, flags);
+    statx.stx_rdev_major = API_VERSION.major().into();
+    statx.stx_rdev_minor = API_VERSION.minor().into();
+    statx.stx_atime = FileStatxTimestamp {
+        tv_sec: 505958400, // must match libsyd!
+        ..Default::default()
+    };
+    statx.stx_ctime = FileStatxTimestamp {
+        tv_sec: -2036448000, // ditto!
+        ..Default::default()
+    };
+    statx.stx_mtime = FileStatxTimestamp {
+        tv_sec: -842745600, // ditto!
+        ..Default::default()
+    };
+}
+
+// Inode = `xxd -r -p < /etc/machine-id | od -An -t u8 | head -n1`
+fn magic_ino() -> u64 {
+    let id = if let Some(id) = std::env::var_os(ENV_ID) {
+        id
+    } else {
+        return 0;
+    };
+
+    if id.len() < 16 {
+        return 0;
+    }
+
+    let id = &id.as_bytes()[..16];
+    let id = if let Ok(id) = HEXLOWER.decode(id) {
+        id
+    } else {
+        return 0;
+    };
+
+    assert_eq!(id.len(), 8);
+    u64::from_ne_bytes(id.try_into().unwrap_or_default())
+}
+
+#[allow(clippy::cognitive_complexity)]
+#[allow(clippy::cast_possible_truncation)]
+fn magic_mode(caps: Capability, flags: Flags) -> u16 {
+    // Start with file type = character device
+    let mut mode: u16 = libc::S_IFCHR as u16;
+
+    // Special bits
+    if flags.contains(Flags::FL_UNSHARE_MOUNT) {
+        mode |= libc::S_ISVTX as u16;
+    }
+    if flags.contains(Flags::FL_UNSHARE_USER) {
+        mode |= libc::S_ISUID as u16;
+    }
+    if flags.contains(Flags::FL_UNSHARE_NET) {
+        mode |= libc::S_ISGID as u16;
+    }
+
+    // Permission bits for owner/group/other
+    if caps.contains(Capability::CAP_READ) {
+        mode |= libc::S_IRUSR as u16;
+    }
+    if caps.contains(Capability::CAP_WRITE) {
+        mode |= libc::S_IWUSR as u16;
+    }
+    if caps.contains(Capability::CAP_EXEC) {
+        mode |= libc::S_IXUSR as u16;
+    }
+    if caps.contains(Capability::CAP_STAT) {
+        mode |= libc::S_IRGRP as u16;
+    }
+    if caps.contains(Capability::CAP_PROXY) {
+        mode |= libc::S_IWGRP as u16;
+    }
+    if caps.contains(Capability::CAP_TPE) {
+        mode |= libc::S_IXGRP as u16;
+    }
+    if caps.contains(Capability::CAP_LOCK) {
+        mode |= libc::S_IROTH as u16;
+    }
+    if caps.contains(Capability::CAP_CRYPT) {
+        mode |= libc::S_IWOTH as u16;
+    }
+    if caps.contains(Capability::CAP_FORCE) {
+        mode |= libc::S_IXOTH as u16;
+    }
+
+    mode
+}
