@@ -151,13 +151,16 @@ mod tests {
                     } else {
                         RulesetStatus::NotEnforced
                     };
+                    let landlock_status = abi.into();
                     println!("Expecting ruleset status {ruleset_status:?}");
+                    println!("Expecting Landlock status {landlock_status:?}");
                     assert!(matches!(
                         ret,
                         Ok(RestrictionStatus {
                             ruleset,
+                            landlock,
                             no_new_privs: true,
-                        }) if ruleset == ruleset_status
+                        }) if ruleset == ruleset_status && landlock == landlock_status
                     ))
                 }
             } else {
