@@ -34,7 +34,7 @@ use crate::{
     landlock::{AccessFs, AccessNet},
     landlock_policy::LandlockPolicy,
     path::XPathBuf,
-    sandbox::{Action, BindMount, Capability},
+    sandbox::{Action, Capability},
 };
 
 // Valid Netlink families.
@@ -71,8 +71,14 @@ const NETLINK_FAMILIES: &[&str] = &[
 pub(crate) struct BindCmd {
     // One of '+', '-', '^'
     pub(crate) op: char,
-    // Parsed BindMount
-    pub(crate) mount: BindMount,
+    // Mount source.
+    pub(crate) src: String,
+    // Mount destination.
+    pub(crate) dst: String,
+    // Mount flags.
+    pub(crate) opt: MsFlags,
+    // Optional filesystem specific data.
+    pub(crate) dat: Option<XPathBuf>,
 }
 
 // Represents a parsed "force" command.
@@ -220,15 +226,6 @@ pub(crate) fn parse_bind_cmd(command: &str) -> Result<BindCmd, Errno> {
 
     match inner(command).finish() {
         Ok(("", (op, src, dst, opt))) => {
-            let src = XPathBuf::from(src);
-            let dst = XPathBuf::from(dst);
-
-            // Validate src/dst.
-            // Deny if dst is not absolute or any ".." in src/dst
-            if dst.is_relative() || src.has_parent_dot() || dst.has_parent_dot() {
-                return Err(Errno::EINVAL);
-            }
-
             // Build MsFlags and dat from opt.
             let mut flags = MsFlags::empty();
             let mut dat = Vec::new();
@@ -272,14 +269,18 @@ pub(crate) fn parse_bind_cmd(command: &str) -> Result<BindCmd, Errno> {
                 Some(XPathBuf::from(dat))
             };
 
-            let mount = BindMount {
-                opt: flags,
+            // Source and destination will be expanded as necessary,
+            // so we perform no further handling here.
+            let src = src.to_string();
+            let dst = dst.to_string();
+
+            Ok(BindCmd {
+                op,
                 src,
                 dst,
                 dat,
-            };
-
-            Ok(BindCmd { op, mount })
+                opt: flags,
+            })
         }
         _ => Err(Errno::EINVAL),
     }
@@ -779,10 +780,10 @@ mod tests {
         let cmd = "bind+/:/:ro";
         let bc = parse_bind_cmd(cmd).unwrap();
         assert_eq!(bc.op, '+');
-        assert_eq!(bc.mount.src, XPathBuf::from("/"));
-        assert_eq!(bc.mount.dst, XPathBuf::from("/"));
-        assert!(bc.mount.opt.contains(MsFlags::MS_RDONLY));
-        assert!(bc.mount.dat.is_none());
+        assert_eq!(bc.src, "/");
+        assert_eq!(bc.dst, "/");
+        assert!(bc.opt.contains(MsFlags::MS_RDONLY));
+        assert!(bc.dat.is_none());
     }
 
     #[test]
@@ -790,10 +791,10 @@ mod tests {
         let cmd = "bind-/mnt/data:/data";
         let bc = parse_bind_cmd(cmd).unwrap();
         assert_eq!(bc.op, '-');
-        assert_eq!(bc.mount.src, XPathBuf::from("/mnt/data"));
-        assert_eq!(bc.mount.dst, XPathBuf::from("/data"));
-        assert!(bc.mount.opt.is_empty());
-        assert!(bc.mount.dat.is_none());
+        assert_eq!(bc.src, "/mnt/data");
+        assert_eq!(bc.dst, "/data");
+        assert!(bc.opt.is_empty());
+        assert!(bc.dat.is_none());
     }
 
     #[test]
@@ -802,12 +803,12 @@ mod tests {
             "bind^overlay:/tmp/target:lowerdir=/tmp/lower,upperdir=/tmp/upper,workdir=/tmp/work";
         let bc = parse_bind_cmd(cmd).unwrap();
         assert_eq!(bc.op, '^');
-        assert_eq!(bc.mount.src, XPathBuf::from("overlay"));
-        assert_eq!(bc.mount.dst, XPathBuf::from("/tmp/target"));
+        assert_eq!(bc.src, "overlay");
+        assert_eq!(bc.dst, "/tmp/target");
         // All three flags are unknown, so go into dat_buf
-        assert!(bc.mount.opt.is_empty());
+        assert!(bc.opt.is_empty());
         assert_eq!(
-            bc.mount.dat.unwrap(),
+            bc.dat.unwrap(),
             XPathBuf::from("lowerdir=/tmp/lower,upperdir=/tmp/upper,workdir=/tmp/work")
         );
     }
@@ -817,12 +818,12 @@ mod tests {
         let cmd = "bind+tmpfs:/tmp:ro,nosuid,size=10M";
         let bc = parse_bind_cmd(cmd).unwrap();
         assert_eq!(bc.op, '+');
-        assert_eq!(bc.mount.src, XPathBuf::from("tmpfs"));
-        assert_eq!(bc.mount.dst, XPathBuf::from("/tmp"));
-        assert!(bc.mount.opt.contains(MsFlags::MS_RDONLY));
-        assert!(bc.mount.opt.contains(MsFlags::MS_NOSUID));
+        assert_eq!(bc.src, "tmpfs");
+        assert_eq!(bc.dst, "/tmp");
+        assert!(bc.opt.contains(MsFlags::MS_RDONLY));
+        assert!(bc.opt.contains(MsFlags::MS_NOSUID));
         // "size=10M" is unknown -> goes into dat_buf
-        assert_eq!(bc.mount.dat.unwrap(), XPathBuf::from("size=10M"));
+        assert_eq!(bc.dat.unwrap(), XPathBuf::from("size=10M"));
     }
 
     #[test]
@@ -832,21 +833,6 @@ mod tests {
         assert_eq!(parse_bind_cmd("bind+:/dst:opt"), Err(Errno::EINVAL));
     }
 
-    #[test]
-    fn test_parse_bind_parse_bind_relative_dst_fails() {
-        // dst does not start with '/', fails
-        assert_eq!(parse_bind_cmd("bind+/src:relative:opt"), Err(Errno::EINVAL));
-    }
-
-    #[test]
-    fn test_parse_bind_parse_bind_parent_dot_fails() {
-        assert_eq!(parse_bind_cmd("bind+/src/../etc:/dst"), Err(Errno::EINVAL));
-        assert_eq!(
-            parse_bind_cmd("bind+/src:/dst/../../tmp"),
-            Err(Errno::EINVAL)
-        );
-    }
-
     #[test]
     fn test_parse_bind_parse_bind_bad_mod_fails() {
         assert_eq!(parse_bind_cmd("bind*src:/dst"), Err(Errno::EINVAL));
