@@ -42,6 +42,7 @@ use crate::{
     alert,
     config::*,
     confine::ExportMode,
+    debug,
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
     fs::{block_signal, closeexcept, seccomp_export_pfc},
@@ -426,8 +427,17 @@ impl ThreadPool {
                             // Wait for grace period before reattempting.
                             std::thread::sleep(MON_GRACE_TIME);
                         }
-                        Err(_) => {
-                            // Caller try_spawn logs an alert level entry about this.
+                        Err(errno) => {
+                            alert!("ctx": "spawn",
+                                "msg": format!("spawn emulator failed: {errno}!"),
+                                "err": errno as i32,
+                                "core_size": self.core_size,
+                                "keep_alive": self.keep_alive);
+
+                            // Be defensive and signal stuck emulator threads to make
+                            // better use of available resources.
+                            self.signal_int();
+
                             // Wait for grace period before reattempting.
                             std::thread::sleep(MON_GRACE_TIME);
                         }
@@ -534,7 +544,7 @@ impl ThreadPool {
     pub(crate) fn try_spawn(
         &self,
         ctx: Option<&ScmpFilterContext>,
-    ) -> SydResult<Option<(SydJoinHandle<()>, bool)>> {
+    ) -> Result<Option<(SydJoinHandle<()>, bool)>, Errno> {
         // Create a new worker if there are no idle threads and the
         // current worker count is lower than the max pool size.
         let worker_count_val = self.worker_data.0.load(Ordering::Relaxed);
@@ -543,14 +553,16 @@ impl ThreadPool {
         let keep_alive = if curr_worker_count < self.core_size {
             // Create a new core worker if current pool size is below
             // core size during the invocation of this function.
-            crate::debug!("ctx": "spawn", "dec": "create_new_core_emulator",
+            debug!("ctx": "spawn",
+                "msg": "creating new core emulator",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size);
             None
         } else if busy_worker_count < curr_worker_count {
             // We have idle threads, no need to spawn a new worker.
-            crate::debug!("ctx": "spawn", "dec": "idle_emulator_exists",
+            debug!("ctx": "spawn",
+                "msg": "idle emulator exists, no need to spawn",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size);
@@ -560,7 +572,8 @@ impl ThreadPool {
             // below the EMU_MAX_SIZE and the pool has been observed to
             // be busy (no idle workers) during the invocation of this
             // function.
-            crate::debug!("ctx": "spawn", "dec": "create_new_idle_emulator",
+            debug!("ctx": "spawn",
+                "msg": "creating new idle emulator",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size,
@@ -569,12 +582,13 @@ impl ThreadPool {
         } else {
             // We cannot spawn anymore workers!
             // Ideally, this should never happen.
-            crate::alert!("ctx": "spawn", "dec": "emulator_capacity_exceeded",
+            alert!("ctx": "spawn",
+                "msg": "emulator capacity exceeded, can not spawn new",
                 "busy_worker_count": busy_worker_count,
                 "curr_worker_count": curr_worker_count,
                 "core_size": self.core_size,
                 "keep_alive": self.keep_alive);
-            return Ok(None);
+            return Err(Errno::ERANGE);
         };
 
         // Try to spawn a new worker.
@@ -596,4 +610,29 @@ impl ThreadPool {
             keep_alive.is_none(),
         )))
     }
+
+    fn signal_int(&self) {
+        // Unblock stuck emulator threads with manual signal.
+        let mut nsig = 0usize;
+        {
+            let (ref lock, ref cvar) = *self.cache.sysint_map.sys_block;
+            let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
+
+            for interrupt in map.iter_mut() {
+                if !interrupt.signal {
+                    interrupt.signal = true;
+                    nsig = nsig.saturating_add(1);
+                }
+            }
+
+            // Notify interrupt thread.
+            cvar.notify_one();
+        }
+
+        alert!("ctx": "spawn",
+            "msg": format!("signaled interrupt to unstuck {nsig} emulators"),
+            "nsig": nsig,
+            "core_size": self.core_size,
+            "keep_alive": self.keep_alive);
+    }
 }
