@@ -2450,15 +2450,18 @@ mod tests {
         }
 
         // Returned paths should exist on filesystem and have exec bit.
+        // ld.so may not have the exec bit on some systems, e.g. i386.
         let bins = proc_executables(Pid::this()).unwrap();
-        for bin in bins {
+        for (idx, bin) in bins.into_iter().enumerate() {
             let path = bin.path;
             let md = std::fs::metadata(&path).expect("path does not exist");
-            let perms = md.permissions().mode();
-            assert!(
-                perms & 0o111 != 0,
-                "file {path} is not executable (mode {perms:o})",
-            );
+            if idx == 0 {
+                let perms = md.permissions().mode();
+                assert!(
+                    perms & 0o111 != 0,
+                    "file {path} is not executable (mode {perms:o})",
+                );
+            }
         }
     }
 
