@@ -202,7 +202,7 @@ struct Riscv64UserRegsStruct {
 ///
 /// This function modifies the architecture-specific register that holds
 /// the system call and the return value.
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<(), Errno> {
     // Quoting seccomp(2):
     // The tracer can skip the system call by changing the system call
@@ -268,9 +268,9 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
         // SAFETY: Get registers.
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = -errno.map(|err| err as i32).unwrap_or(0);
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         match arch {
             SCMP_ARCH_X86_64 => {
                 regs.x64.orig_rax = u64::MAX;
@@ -394,7 +394,7 @@ pub fn ptrace_skip_syscall(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<
 ///
 /// This function modifies the architecture-specific register that holds
 /// the return value.
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<(), Errno> {
     #[cfg(target_arch = "x86_64")]
     {
@@ -419,7 +419,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
         // Determine the value to set.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             // Error case: Set the error code as a negative value.
             -(e as i64)
@@ -428,8 +428,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             0
         };
 
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_possible_truncation)]
         match arch {
             SCMP_ARCH_X86_64 | SCMP_ARCH_X32 => regs.x64.rax = rval as u64,
             SCMP_ARCH_X86 => regs.x32.eax = (rval as i32) as u32,
@@ -455,7 +455,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         }
 
         // Determine the value to set.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             // Error case: Set the error code as a negative value.
             -(e as i32)
@@ -507,14 +507,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             SCMP_ARCH_AARCH64 => {
                 // SAFETY: aarch64 personality
                 let regs_ref = unsafe { &mut regs.aarch64 };
-                #[allow(clippy::arithmetic_side_effects)]
+                #[expect(clippy::arithmetic_side_effects)]
                 let rval = if let Some(e) = errno {
                     -(e as i64) // Error case
                 } else {
                     0 // Success case
                 };
 
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_sign_loss)]
                 {
                     // Set return value in X0.
                     regs_ref.regs[0] = rval as u64;
@@ -523,14 +523,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             SCMP_ARCH_ARM => {
                 // SAFETY: arm personality
                 let regs_ref = unsafe { &mut regs.arm };
-                #[allow(clippy::arithmetic_side_effects)]
+                #[expect(clippy::arithmetic_side_effects)]
                 let rval = if let Some(e) = errno {
                     -(e as i32) // Error case
                 } else {
                     0 // Success case
                 };
 
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::cast_sign_loss)]
                 {
                     // Set return value in R0.
                     regs_ref.uregs[0] = rval as u32;
@@ -578,14 +578,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         })?;
 
         // Modify the return value in R0.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i32) // Error case.
         } else {
             0 // Success case.
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             regs.uregs[0] = rval as u32;
         }
@@ -622,7 +622,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         // SAFETY: PTRACE_GETREGS returned success.
         let mut regs = unsafe { regs.assume_init() };
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as c_long) // Error case
         } else {
@@ -692,7 +692,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         const REG_A3: usize = REG_A0 + 3;
 
         // Modify the return value.
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         if matches!(arch, SCMP_ARCH_MIPS | SCMP_ARCH_MIPSEL) {
             if let Some(e) = errno {
                 // Error case
@@ -751,14 +751,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
         })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i64) // Error case
         } else {
             0 // Success case
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // Modify the return value in A0.
             regs.a0 = rval as u64;
@@ -809,14 +809,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
         })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i64) // Error case
         } else {
             0 // Success case
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // Modify the return value in GPR2
             regs.gprs[2] = rval as u64;
@@ -859,8 +859,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
         let mut regs = unsafe { regs.assume_init() };
 
         // Modify the return value in GPR3.
-        #[allow(clippy::arithmetic_side_effects)]
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::arithmetic_side_effects)]
+        #[expect(clippy::cast_sign_loss)]
         if let Some(e) = errno {
             if (regs.trap & 0xfff0) == 0x3000 {
                 // SCV case: Error value is negated.
@@ -926,8 +926,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
                 let regs = unsafe { &mut regs.ppc64 };
 
                 // Modify the return value in GPR3.
-                #[allow(clippy::arithmetic_side_effects)]
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::arithmetic_side_effects)]
+                #[expect(clippy::cast_sign_loss)]
                 if let Some(e) = errno {
                     if (regs.trap & 0xfff0) == 0x3000 {
                         // SCV case: Error value is negated.
@@ -951,8 +951,8 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
                 let regs = unsafe { &mut regs.ppc32 };
 
                 // Modify the return value in GPR3.
-                #[allow(clippy::arithmetic_side_effects)]
-                #[allow(clippy::cast_sign_loss)]
+                #[expect(clippy::arithmetic_side_effects)]
+                #[expect(clippy::cast_sign_loss)]
                 if let Some(e) = errno {
                     if (regs.trap & 0xfff0) == 0x3000 {
                         // SCV case: Error value is negated.
@@ -1005,14 +1005,14 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
             libc::ptrace(PTRACE_GETREGSET, pid.as_raw(), libc::NT_PRSTATUS, &mut io)
         })?;
 
-        #[allow(clippy::arithmetic_side_effects)]
+        #[expect(clippy::arithmetic_side_effects)]
         let rval = if let Some(e) = errno {
             -(e as i64) // Error case
         } else {
             0 // Success case
         };
 
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         {
             // Modify the return value in regs[4].
             regs.regs[4] = rval as u64;
@@ -1048,7 +1048,7 @@ pub fn ptrace_set_return(pid: Pid, arch: u32, errno: Option<Errno>) -> Result<()
 
 /// Retrieve the system call return code from the tracee and determine
 /// if it indicates an error or success.
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
     #[cfg(target_arch = "x86_64")]
     {
@@ -1072,7 +1072,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         // SAFETY: Get registers.
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         let val: i64 = match arch {
             SCMP_ARCH_X86_64 | SCMP_ARCH_X32 => {
                 // SAFETY: Keep it as 64 bits, interpret as signed.
@@ -1447,7 +1447,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
         })?;
 
         // On PPC, the return value is always in gpr[3].
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         let (r3, scv, ccr) = match arch {
             SCMP_ARCH_PPC64 | SCMP_ARCH_PPC64LE => {
                 // SAFETY: ppc64 personality
@@ -1549,8 +1549,7 @@ pub fn ptrace_get_error(pid: Pid, arch: u32) -> Result<Option<Errno>, Errno> {
 ///
 /// This function modifies the architecture-specific register that holds
 /// the syscall number.
-#[allow(clippy::cast_possible_wrap)]
-#[allow(unused_variables)]
+#[allow(unused)]
 pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno> {
     #[cfg(target_arch = "x86_64")]
     {
@@ -1576,7 +1575,7 @@ pub fn ptrace_set_syscall(pid: Pid, arch: u32, sysno: u64) -> Result<(), Errno>
         // SAFETY: Get registers.
         Errno::result(unsafe { ptrace(PTRACE_GETREGSET, pid.as_raw(), NT_PRSTATUS, &mut io) })?;
 
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         match arch {
             SCMP_ARCH_X86_64 => regs.x64.orig_rax = sysno,
             SCMP_ARCH_X32 => regs.x64.orig_rax = sysno | (X32_SYSCALL_BIT as u64),
@@ -1912,8 +1911,8 @@ fn check_negated_errno(val: i64) -> Option<Errno> {
     // (somewhat standard across Linux).
     // If val is in the range -4095..=-1, it's an error code.
     const MIN_ERRNO: i64 = -4095;
-    #[allow(clippy::arithmetic_side_effects)]
-    #[allow(clippy::cast_possible_truncation)]
+    #[expect(clippy::arithmetic_side_effects)]
+    #[expect(clippy::cast_possible_truncation)]
     if (MIN_ERRNO..0).contains(&val) {
         // We flip the sign to get the positive errno.
         Some(Errno::from_raw((-val) as i32))
@@ -2079,7 +2078,7 @@ impl ptrace_syscall_info {
         };
 
         // SAFETY: In libseccomp we trust.
-        #[allow(clippy::cast_possible_truncation)]
+        #[expect(clippy::cast_possible_truncation)]
         let ptr = unsafe { seccomp_syscall_resolve_num_arch(self.arch, nr as i32) };
 
         // Check for NULL.
