@@ -69,6 +69,15 @@ bitflags! {
     pub(crate) struct SysFlags: u8 {
         /// Whether if it's ok for the path to be empty.
         const EMPTY_PATH = 1 << 0;
+        /// The system call should be checked for /dev/syd access.
+        const CHECK_MAGIC = 1 << 1;
+    }
+}
+
+impl SysFlags {
+    /// Return true if syscall should be checked for /dev/syd access.
+    pub fn is_check_magic(self) -> bool {
+        self.contains(Self::CHECK_MAGIC)
     }
 }
 
@@ -234,7 +243,6 @@ impl RemoteProcess {
         sandbox: &SandboxGuard,
         args: [u64; 6],
         arg: SysArg,
-        mut magic: bool,
         request: Option<&UNotifyEventRequest>,
     ) -> Result<(CanonicalPath<'b>, bool, bool, bool), Errno> {
         let orig = match arg.path {
@@ -250,11 +258,9 @@ impl RemoteProcess {
         };
         let mut doterr = false;
 
-        // magic is both an in and out variable.
-        // in=t: check for magic path.
-        // out=t: path is magic path.
-        let check_magic = magic;
-        magic = false;
+        // Should we check for magic path?
+        let check_magic = arg.flags.is_check_magic();
+        let mut is_magic = false;
 
         let mut empty_path = false;
         let canonical_path = if let Some(path) = orig {
@@ -277,7 +283,7 @@ impl RemoteProcess {
             }
 
             if check_magic && path.is_magic() {
-                magic = true;
+                is_magic = true;
                 CanonicalPath::new_magic(path)
             } else if empty_path || path.is_dot() {
                 let dirfd = if let Some(idx) = arg.dirfd {
@@ -358,7 +364,31 @@ impl RemoteProcess {
                     None
                 };
 
-                safe_canonicalize(self.pid, fd, &path, arg.fsflags, Some(sandbox.deref()))?
+                let mut path =
+                    safe_canonicalize(self.pid, fd, &path, arg.fsflags, Some(sandbox.deref()))?;
+
+                // Convert /proc/${pid} to /proc/self as necessary.
+                // This is never used in syscalls only in access check.
+                if path.base.is_empty() {
+                    if let Some(p) = path.abs().split_prefix(b"/proc") {
+                        let mut buf = itoa::Buffer::new();
+                        let pid = buf.format(self.pid.as_raw());
+                        if let Some(p) = p.split_prefix(pid.as_bytes()) {
+                            let mut pdir = XPathBuf::from("/proc/self");
+                            // SAFETY: We do not use XPathBuf::push here,
+                            // and allow slash in path. This is safe against
+                            // path traversals because the path is guaranteed
+                            // to be a canonicalized path.
+                            pdir.append_byte(b'/');
+                            pdir.append_bytes(p.as_bytes());
+
+                            // SAFETY: path.base is checked to be empty.
+                            path.set_abs(pdir);
+                        }
+                    }
+                }
+
+                path
             }
         } else {
             // SAFETY: SysArg.path is None asserting dirfd is Some.
@@ -400,7 +430,7 @@ impl RemoteProcess {
             }
         };
 
-        if !magic && arg.path.is_some() {
+        if !is_magic && arg.path.is_some() {
             // SAFETY: Deny access to critical and/or suspicious paths.
             canonical_path.abs().check(
                 self.pid,
@@ -411,7 +441,7 @@ impl RemoteProcess {
             )?;
         }
 
-        Ok((canonical_path, magic, doterr, empty_path))
+        Ok((canonical_path, is_magic, doterr, empty_path))
     }
 
     /// Allocate and read a `Zeroizing` buffer from remote process's memory with `process_vm_readv()`.
@@ -1025,7 +1055,7 @@ impl UNotifyEventRequest {
         fd: Fd,
         flags: Flags,
         op: u8,
-        verbose: bool,
+        log_scmp: bool,
     ) -> Result<(), Errno> {
         // SAFETY: Deny sending/receiving file descriptors referring to
         // 1. Directories (pledge does the same).
@@ -1046,7 +1076,7 @@ impl UNotifyEventRequest {
             _ => return Ok(()),
         };
 
-        if verbose {
+        if log_scmp {
             error!("ctx": "trusted_scm_rights", "sys": op2name(op),
                 "msg": format!("SCM_RIGHTS for unsafe file type `{ftyp:?}' blocked"),
                 "tip": emsg, "req": &self);
@@ -1535,13 +1565,12 @@ impl UNotifyEventRequest {
         &self,
         sandbox: &SandboxGuard,
         arg: SysArg,
-        magic: bool,
     ) -> Result<(CanonicalPath<'b>, bool, bool), Errno> {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated by read_path.
         let (path, magic, doterr, empty_path) =
-            process.read_path(sandbox, self.scmpreq.data.args, arg, magic, Some(self))?;
+            process.read_path(sandbox, self.scmpreq.data.args, arg, Some(self))?;
 
         // Determine FD-only system calls.
         // We return EACCES rather than ENOENT for these.
