@@ -76,11 +76,14 @@ use nix::{
     pty::{grantpt, unlockpt},
     sched::{unshare, CloneFlags},
     sys::{
+        prctl::{set_name, set_pdeathsig},
+        signal::Signal,
         socket::{bind, socket, AddressFamily, SockFlag, SockType, SockaddrIn, SockaddrIn6},
+        stat::{umask, Mode},
         wait::{Id, WaitPidFlag},
     },
     time::{clock_gettime, ClockId},
-    unistd::{fork, getgid, getpid, getuid, isatty, sethostname, ForkResult, Pid},
+    unistd::{fork, getgid, getpid, getuid, isatty, sethostname, setsid, ForkResult, Pid},
 };
 use sendfd::SendWithFd;
 use syd::{
@@ -109,10 +112,15 @@ use syd::{
     warn, IgnoreSignalOpts,
 };
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -122,9 +130,12 @@ static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
 syd::main! {
     use lexopt::prelude::*;
 
-    // Initialize logging..
+    // Initialize logging.
     log_init(LogLevel::Warn, Some(libc::STDERR_FILENO))?;
 
+    // Set process name, ignore errors.
+    let _ = set_name(c"syd");
+
     // Parse CLI options.
     //
     // Note, option parsing is POSIXly correct:
@@ -235,7 +246,9 @@ syd::main! {
             Some(val) => env::set_var("SYD_RUST_BACKTRACE", val),
             None => env::remove_var("SYD_RUST_BACKTRACE"),
         };
-        env::set_var("RUST_BACKTRACE", "0");
+        if env::var_os(ENV_SKIP_SCMP).is_none() {
+            env::set_var("RUST_BACKTRACE", "0");
+        }
         env::set_var(ENV_RAND, get_at_random_hex(false));
         env::var(ENV_RAND).unwrap()
     };
@@ -374,7 +387,7 @@ syd::main! {
                 syd_info(false)?;
                 return Ok(ExitCode::SUCCESS);
             }
-            Short('v') | Long("verbose") => sandbox.verbose = true,
+            Short('v') | Long("verbose") => sandbox.increase_verbosity(),
             Long("el") => {
                 set_sigpipe_dfl()?;
                 stdout().write_all(SYD_EL.as_bytes())?;
@@ -640,6 +653,16 @@ syd::main! {
             cmd.arg("-y");
             cmd.arg(buf.format(ws));
         }
+        // SAFETY:
+        // Set umask(2) to a sane value.
+        // To let syd-pty(1) restore terminal at exit:
+        // 1. Do not set parent-death-signal.
+        // 2. Do not call setsid(2).
+        unsafe { cmd.pre_exec(|| {
+            let _ = set_name(c"syd_pty");
+            umask(Mode::from_bits_truncate(0o777));
+            Ok(())
+        }) };
         if let Err(error) = cmd.spawn() {
             let errno = err2no(&error);
             error!("ctx": "spawn_pty",
@@ -697,7 +720,6 @@ syd::main! {
         } else {
             std::process::Command::new("syd-tor")
         };
-        cmd.process_group(0);
         cmd.stdin(Stdio::inherit());
         cmd.stdout(Stdio::inherit());
         cmd.env_clear();
@@ -726,6 +748,17 @@ syd::main! {
             cmd.arg("-o");
             cmd.arg(addr);
         }
+        // SAFETY:
+        // 1. Set parent death signal to SIGKILL.
+        // 2. Create a new session.
+        // 3. Set umask(2) to a sane value.
+        unsafe { cmd.pre_exec(|| {
+            let _ = set_name(c"syd_tor");
+            set_pdeathsig(Some(Signal::SIGKILL))?;
+            setsid()?;
+            umask(Mode::from_bits_truncate(0o777));
+            Ok(())
+        }) };
         if let Err(error) = cmd.spawn() {
             let errno = err2no(&error);
             error!("ctx": "spawn_tor",
@@ -767,7 +800,7 @@ syd::main! {
         ) {
             Ok(code) => Ok(ExitCode::from(code)),
             Err(error) => {
-                let errno = Errno::last();
+                let errno = error.errno().unwrap_or(Errno::ENOSYS);
                 eprintln!("{error:?}");
                 Ok(ExitCode::from(u8::try_from(errno as i32).unwrap_or(127)))
             }
@@ -1093,7 +1126,7 @@ syd::main! {
         #[cfg(feature = "prof")]
         "gettid",
     ];
-    for name in allow_call {
+    for name in allow_call.iter().chain(VDSO_SYSCALLS) {
         if let Ok(syscall) = ScmpSyscall::from_name(name) {
             ctx.add_rule(ScmpAction::Allow, syscall)?;
         }
@@ -1174,9 +1207,7 @@ fn help() {
     println!("{c_blue}License:{c_res} {c_yll}GPL-3.0-only{c_res}");
     println!();
     println!("{c_green}$ syd [-acefhlmpqxEPV] [--] {{command [arg...]}}{c_res}");
-    println!("  {c_bold}Run a program under syd.{c_res}");
-    println!("{c_green}$ syd [-acefhlmpqxEPV] [--] {{library.so}}{c_res}");
-    println!("  {c_bold}Load a library under syd.{c_res}");
+    println!("  {c_bold}Run a program under Syd.{c_res}");
     println!("{c_green}$ syd --api{c_res}");
     println!("  {c_bold}Print syd(2) API specification.{c_res}");
     println!("{c_green}$ syd --check{c_res}");
