@@ -239,6 +239,13 @@ macro_rules! assert_status_broken_pipe {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_busy {
+    ($status:expr) => {
+        $crate::assert_status_code!($status, libc::EBUSY);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_hidden {
     ($status:expr) => {
@@ -1192,8 +1199,6 @@ pub fn syd() -> Syd {
     // Set logging level to warning.
     // Allow user to override.
     cmd.env(ENV_LOG, env::var(ENV_LOG).unwrap_or("warn".to_string()));
-    // Disallow backtraces.
-    cmd.env("RUST_BACKTRACE", "0");
     // Set log/verbose to false to avoid noisy test logs.
     cmd.m("log/verbose:0");
     // Quiet TTY output to avoid noisy test logs.
