@@ -146,7 +146,7 @@ const fn zero_init_timespec() -> libc::timespec {
     // SAFETY: `std::mem::MaybeUninit::zeroed()` is not yet a const fn
     // (https://github.com/rust-lang/rust/issues/91850) so we will instead initialize an array of
     // the appropriate size to zero and then transmute it to a timespec value.
-    unsafe { std::mem::transmute([0u8; std::mem::size_of::<libc::timespec>()]) }
+    unsafe { std::mem::transmute([0u8; size_of::<libc::timespec>()]) }
 }
 
 #[derive(Debug, Clone, Copy)]
