@@ -10,12 +10,15 @@ use std::{
 };
 
 use btoi::btoi;
-use libloading::os::unix::Symbol;
+use memchr::arch::all::is_prefix;
 use nix::{
     errno::Errno,
     libc,
-    sys::signal::{kill, sigprocmask, SigSet, SigmaskHow, Signal},
-    unistd::{close, getpid, read, setsid, tcsetpgrp, write},
+    sys::{
+        prctl::{set_name, set_pdeathsig},
+        signal::{kill, sigprocmask, SigSet, SigmaskHow, Signal},
+    },
+    unistd::{close, execvp, getpid, read, setsid, tcsetpgrp, write},
 };
 
 use crate::{
@@ -24,10 +27,7 @@ use crate::{
     confine::{confine_scmp_ioctl, confine_scmp_pwritev2, safe_drop_cap},
     landlock::{CompatLevel, Compatible, RestrictSelfFlags, Ruleset, RulesetAttr, Scope},
     retry::retry_on_eintr,
-    unshare::{
-        error::ErrorCode as Err,
-        run::{ChildInfo, Exe},
-    },
+    unshare::{error::ErrorCode as Err, run::ChildInfo},
 };
 
 unsafe fn fail_errno(code: Err, errno: i32) -> ! {
@@ -70,9 +70,38 @@ macro_rules! fail_errno_safe {
 
 #[expect(clippy::cognitive_complexity)]
 pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
+    // Set process name, ignore errors.
+    let _ = set_name(c"syd_exec");
+
     // SAFETY: arg is a valid ChildInfo structure.
     let mut child: Box<ChildInfo> = unsafe { Box::from_raw(arg as *mut ChildInfo) };
 
+    if let Some(&sig) = child.cfg.death_sig.as_ref() {
+        if let Err(errno) = set_pdeathsig(Some(sig)) {
+            fail_errno_safe!(child, Err::ParentDeathSignal, errno as i32);
+        }
+    }
+
+    if child.cfg.restore_sigmask {
+        // Reset blocking signals.
+        // Step 1: Reset the signal mask using pthread_sigmask.
+        unsafe {
+            let mut sigmask: libc::sigset_t = mem::zeroed();
+            libc::sigemptyset(&raw mut sigmask);
+            libc::pthread_sigmask(libc::SIG_SETMASK, &raw const sigmask, ptr::null_mut());
+        }
+        // Step 2: Unblock all signals using sigprocmask.
+        let sigmask = SigSet::all();
+        if let Err(errno) = sigprocmask(SigmaskHow::SIG_UNBLOCK, Some(&sigmask), None) {
+            fail_errno_safe!(child, Err::ResetSignal, errno as i32);
+        }
+
+        // Reset all signals to their default dispositions.
+        if let Err(errno) = crate::reset_signals() {
+            fail_errno_safe!(child, Err::ResetSignal, errno as i32);
+        }
+    }
+
     // Restriction 0: Change controlling terminal to PTY as necessary.
     if let Some(pty_fd) = child.pty_fd.take() {
         // SAFETY: pty_fd is a valid FD.
@@ -171,8 +200,8 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
     // SAFETY: Clean Syd environment variables from process environment.
     // Note, we have just used ENV_LOG_FD above and do not need it anymore.
     for (key, _) in env::vars_os() {
-        if key.as_bytes().starts_with(b"CARGO_BIN_EXE_syd")
-            || (key.as_bytes().starts_with(b"SYD_") && !key.as_bytes().starts_with(b"SYD_TEST_"))
+        if is_prefix(key.as_bytes(), b"CARGO_BIN_EXE_syd")
+            || (is_prefix(key.as_bytes(), b"SYD_") && !is_prefix(key.as_bytes(), b"SYD_TEST_"))
         {
             env::remove_var(key);
         }
@@ -187,34 +216,6 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
     drop(child.seccomp_pipefd.0 .1);
     drop(child.seccomp_pipefd.1 .0);
 
-    if let Some(&sig) = child.cfg.death_sig.as_ref() {
-        if let Err(errno) = Errno::result(unsafe {
-            libc::prctl(libc::PR_SET_PDEATHSIG, sig as libc::c_ulong, 0, 0, 0)
-        }) {
-            fail_errno_safe!(child, Err::ParentDeathSignal, errno as i32);
-        }
-    }
-
-    if child.cfg.restore_sigmask {
-        // Reset blocking signals.
-        // Step 1: Reset the signal mask using pthread_sigmask.
-        unsafe {
-            let mut sigmask: libc::sigset_t = mem::zeroed();
-            libc::sigemptyset(&raw mut sigmask);
-            libc::pthread_sigmask(libc::SIG_SETMASK, &raw const sigmask, ptr::null_mut());
-        }
-        // Step 2: Unblock all signals using sigprocmask.
-        let sigmask = SigSet::all();
-        if let Err(errno) = sigprocmask(SigmaskHow::SIG_UNBLOCK, Some(&sigmask), None) {
-            fail_errno_safe!(child, Err::ResetSignal, errno as i32);
-        }
-
-        // Reset all signals to their default dispositions.
-        if let Err(errno) = crate::reset_signals() {
-            fail_errno_safe!(child, Err::ResetSignal, errno as i32);
-        }
-    }
-
     if let Some(callback) = &child.pre_exec {
         if let Err(errno) = callback() {
             fail_errno_safe!(child, Err::PreExec, errno as i32);
@@ -352,18 +353,6 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
         fail_safe!(child, Err::CapSet);
     }
 
-    match child.exe {
-        Exe::Library(lib) => unsafe {
-            let fun: Symbol<unsafe extern "C" fn() -> i32> = match lib.get(b"syd_main") {
-                Ok(fun) => fun,
-                Err(_) => nix::libc::_exit(nix::libc::EINVAL),
-            };
-            nix::libc::_exit(fun());
-        },
-        Exe::Program((filename, ref args)) => {
-            let args = &args[..];
-            unsafe { libc::execvp(filename, args.as_ptr()) };
-            fail_safe!(child, Err::Exec);
-        }
-    }
+    let Err(errno) = execvp(&child.exe_file, &child.exe_args);
+    fail_errno_safe!(child, Err::Exec, errno as i32);
 }
