@@ -10,12 +10,20 @@ use std::{net::IpAddr, process::ExitCode};
 
 use libc::{res_init, AF_INET, AF_INET6};
 use nix::errno::Errno;
-use syd::dns::{lookup_addr, resolve_host, resolve_rand};
+use syd::{
+    dns::{lookup_addr, resolve_host, resolve_rand},
+    path::XPathBuf,
+};
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -92,7 +100,7 @@ syd::main! {
                 return Err(Errno::EINVAL.into());
             }
         };
-        let name = lookup_addr(addr)?;
+        let name: XPathBuf = lookup_addr(addr)?.into();
         println!("{name}");
     } else if opt_rand {
         let addr = resolve_rand(&hostname, family)?;
