@@ -13,6 +13,21 @@ use std::{
 
 use nix::{fcntl::OFlag, sys::stat::Mode, unistd::Uid};
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     syd::set_sigpipe_dfl()?;
 
@@ -28,7 +43,7 @@ syd::main! {
     // Set up logging.
     let mut buf = itoa::Buffer::new();
     let logf = home.join(b".emacs.d/syd.log");
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let file = nix::fcntl::open(
         &logf,
         OFlag::O_CREAT | OFlag::O_APPEND,
