@@ -21,10 +21,15 @@ use nix::{
 };
 use syd::{path::XPathBuf, proc::proc_environ_read, syslog::LogLevel, wordexp::WordExp};
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
