@@ -1,6 +1,6 @@
 //
 // Syd: rock-solid application kernel
-// src/utils/syd-load.rs: Run a command under Memory-Deny-Write-Execute protections
+// src/utils/syd-mdwe.rs: Run a command under Memory-Deny-Write-Execute protections
 //
 // Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 //
@@ -20,10 +20,15 @@ use syd::{
     confine::{confine_mdwe, confine_scmp_wx, run_cmd},
 };
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
