@@ -16,6 +16,21 @@ use data_encoding::HEXLOWER;
 use nix::errno::Errno;
 use syd::hash::HashAlgorithm;
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
@@ -64,7 +79,7 @@ syd::main! {
             }
         }
         Some(path) => {
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let file = BufReader::new(File::open(path)?);
             if opt_bino {
                 // Binary output for file input
