@@ -25,15 +25,20 @@ use syd::{
     config::proc_init_simple,
     confine::print_seccomp_architectures,
     fs::unix_inodes,
-    ioctl::{ioctl_map_get, ioctl_names_get, Ioctl},
+    ioctl::{Ioctl, IoctlMap},
     proc::proc_unix_inodes,
     wildmatch::inamematch,
 };
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -202,9 +207,10 @@ syd::main! {
             }
         };
     } else if opt_ioctl {
+        let ioctl = IoctlMap::new(false);
         return match sysarg.parse::<Ioctl>() {
             Ok(num) => {
-                if let Some(names) = ioctl_names_get(num, opt_arch) {
+                if let Some(names) = ioctl.get_names(num, opt_arch) {
                     for name in names {
                         println!("{name}\t{num}");
                     }
@@ -215,7 +221,7 @@ syd::main! {
             }
             Err(_) => {
                 let glob = sysarg.to_str().ok_or(Errno::EINVAL)?;
-                let list = ioctl_map_get(opt_arch).ok_or(Errno::EINVAL)?;
+                let list = ioctl.get_map(opt_arch).ok_or(Errno::EINVAL)?;
 
                 let mut ok = false;
                 for (num, names) in list {
