@@ -46,14 +46,13 @@ use crate::{
     cookie::safe_memfd_create,
     debug,
     err::err2no,
-    fs::{
-        pidfd_send_signal, safe_clone, safe_copy_if_exists, seal_memfd, set_cloexec, set_nonblock,
-    },
+    fs::{pidfd_send_signal, safe_clone, seal_memfd_all, set_cloexec, set_nonblock},
     get_user_home, get_user_name,
     hash::SydHashSet,
     landlock::RulesetStatus,
     landlock_policy::LandlockPolicy,
     log::contains_ascii_unprintable,
+    lookup::safe_copy_if_exists,
     path::PATH_MAX,
     unshare::{GidMap, UidMap},
     xpath, XPath, XPathBuf,
@@ -336,7 +335,7 @@ impl WordExp<'_> {
         file.write_all(b"\nshift\nprintf '%s ' \"$@\"\n")?;
 
         // Seal memfd for future writes.
-        seal_memfd(&file)?;
+        seal_memfd_all(&file)?;
         debug!("ctx": "expand",
             "msg": format!("sealed memory-file {} against grows, shrinks and writes",
                 file.as_raw_fd()));
@@ -555,6 +554,10 @@ impl WordExp<'_> {
     #[expect(clippy::cognitive_complexity)]
     #[expect(clippy::disallowed_methods)]
     pub fn confine() {
+        if env::var_os(ENV_SKIP_SCMP).is_some() {
+            return;
+        }
+
         // SAFETY: Determine user HOME directory.
         // This will be confined by Landlock.
         let uid = Uid::current();
