@@ -216,7 +216,7 @@ impl<'a> WorkerCache<'a> {
     }
 
     // Add a restarting signal.
-    pub(crate) fn add_sig_restart(&self, request_tgid: Pid, sig: libc::c_int) {
+    pub(crate) fn add_sig_restart(&self, request_tgid: Pid, sig: libc::c_int) -> Result<(), Errno> {
         let mut map = self
             .sysint_map
             .sig_restart
@@ -224,13 +224,16 @@ impl<'a> WorkerCache<'a> {
             .unwrap_or_else(|err| err.into_inner());
         if let Some(set) = map.get_mut(&request_tgid) {
             set.add(sig);
-            return;
+            return Ok(());
         }
 
         let mut set = SydSigSet::new(0);
         set.add(sig);
 
+        map.try_reserve(1).or(Err(Errno::ENOMEM))?;
         map.insert(request_tgid, set);
+
+        Ok(())
     }
 
     // Delete a restarting signal.
